class HoldingDTO:
    def __init__(self, collection_id: str, asset_id: str, address: str, quantity: int):
        self.collection_id: str = collection_id
        self.asset_id: str = asset_id
        self.address: str = address
        self.quantity: int = quantity

    def __repr__(self) -> str:
        if not self.collection_id:
            return f"{self.asset_id} x {self.quantity}"

        return f"{self.collection_id}/{self.asset_id} x {self.quantity}"
